// -*-c++-*-

/*
 *Copyright:

 Copyright (C) Hidehisa AKIYAMA, Hiroki SHIMORA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 3, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "bhv_basic_move.h"

#include "strategy.h"

#include "bhv_basic_tackle.h"

#include <rcsc/action/basic_actions.h>
#include <rcsc/action/body_go_to_point.h>
#include "puppets/body_goalie_go_to_point.h"
#include <rcsc/action/body_intercept.h>
#include <rcsc/action/body_intercept2009_eval_max.h>
#include <rcsc/action/neck_turn_to_ball_or_scan.h>
#include <rcsc/action/neck_turn_to_low_conf_teammate.h>

#include <rcsc/player/player_agent.h>
#include <rcsc/player/debug_client.h>
#include <rcsc/player/intercept_table.h>

#include <rcsc/common/logger.h>
#include <rcsc/common/server_param.h>

#include "neck_offensive_intercept_neck.h"

#if 1
#define USE_LOCAL_POSITIONING
#endif

#ifdef USE_LOCAL_POSITIONING
#include "puppets/local_positioning.h"
#endif

using namespace rcsc;

/*-------------------------------------------------------------------*/
/*!

 */
bool
Bhv_BasicMove::execute( PlayerAgent * agent )
{
    dlog.addText( Logger::TEAM,
                  __FILE__": Bhv_BasicMove" );

    //-----------------------------------------------
    // tackle
    if ( Bhv_BasicTackle( 0.8, 80.0 ).execute( agent ) )
    {
        return true;
    }

    const WorldModel & wm = agent->world();
    /*--------------------------------------------------------*/
    // chase ball
    const int self_min = wm.interceptTable()->selfReachCycle();
    const int mate_min = wm.interceptTable()->teammateReachCycle();
#if 0
    const int opp_min = wm.interceptTable()->opponentReachCycle();
#endif
    const AbstractPlayerObject * opp = wm.interceptTable()->fastestOpponent();
    const Vector2D opp_pos = (opp? opp->pos() : wm.ball().pos());


    if ( ! wm.existKickableTeammate()
         && ( self_min <= 3
              || ( self_min <= mate_min
#if 0
                   && self_min < opp_min + 3
#elif 0
                   && ( (opp_pos.x - 5.0 < wm.self().pos().x
                         && wm.self().pos().x < opp_pos.x + 1.0
                         && std::fabs( opp_pos.x - wm.self().pos().y ) < 7.0 )
                        || self_min < opp_min + 3
                        || wm.ball().pos().x > wm.ourDefenseLineX() + 5.0 )
#else
                   // don't check self_min opp_min
#endif
                   )
              )
         )
    {
        dlog.addText( Logger::TEAM,
                      __FILE__": intercept" );
#if 0
        Body_Intercept().execute( agent );
#else
        Body_Intercept2009EvalMax().execute( agent );
#endif
        agent->setNeckAction( new Neck_OffensiveInterceptNeck() );

        return true;
    }

#ifdef USE_LOCAL_POSITIONING
    Vector2D original_home_pos
        = Strategy::i().getPosition( wm.self().unum() );
    Vector2D home_pos = original_home_pos;
//  agent->debugClient().addLine( wm.self().pos(), original_home_pos );
    if ( original_home_pos.dist( wm.self().pos() ) < 10.0 )
    {
        LocalPositioning local_pos;
        home_pos = local_pos.getLocalAdjustmentPoint( agent, home_pos );
//      agent->debugClient().addLine( wm.self().pos(), home_pos );
//      agent->debugClient().addLine( original_home_pos, home_pos );
    }
    const Vector2D target_point = home_pos;
#else
    const Vector2D target_point
        = Strategy::i().getPosition( wm.self().unum() );
#endif
    const double dash_power = this->getDashPower( agent );

    double dist_thr = wm.ball().distFromSelf() * 0.1;
#ifdef USE_LOCAL_POSITIONING
    if ( home_pos.dist( original_home_pos ) >= 0.1 )
    {
        dist_thr = std::max( original_home_pos.dist( home_pos ) * 0.3,
                             dist_thr );
    }
#endif
    if ( dist_thr < 1.0 ) dist_thr = 1.0;

    agent->debugClient().addMessage( "BasicMove%.0f", dash_power );
#if 0
    agent->debugClient().setTarget( home_pos );
    agent->debugClient().addCircle( home_pos, dist_thr );
#endif

    agent->debugClient().addMessage( "BasicMove%.0f", dash_power );
#if 0
    agent->debugClient().setTarget( target_point );
    agent->debugClient().addCircle( target_point, dist_thr );
#endif

#if 0
    if ( ! Body_GoToPoint( target_point, dist_thr, dash_power
                           ).execute( agent ) )
#else
    if ( ! Body_GoalieGoToPoint( target_point, dist_thr, dash_power
                           ).execute( agent ) )
#endif
    {
        Body_TurnToBall().execute( agent );
    }

    if ( wm.existKickableOpponent()
         && wm.ball().distFromSelf() < 18.0 )
    {
        agent->setNeckAction( new Neck_TurnToBall() );
    }
    else
    {
        agent->setNeckAction( new Neck_TurnToBallOrScan() );
    }

    return true;
}

/*-------------------------------------------------------------------*/
/*!

*/
double
Bhv_BasicMove::getDashPower( const rcsc::PlayerAgent * agent )
{
    static bool s_recover_mode = false;

    const rcsc::WorldModel & wm = agent->world();

    if ( wm.self().staminaModel().capacityIsEmpty() )
    {
        return std::min( ServerParam::i().maxDashPower(),
                         wm.self().stamina() + wm.self().playerType().extraStamina() );
    }

    const int self_min = wm.interceptTable()->selfReachCycle();
    const int mate_min = wm.interceptTable()->teammateReachCycle();
    const int opp_min = wm.interceptTable()->opponentReachCycle();

    // check recover
    if ( wm.self().staminaModel().capacityIsEmpty() )
    {
        s_recover_mode = false;
    }
    else if ( wm.self().stamina() < rcsc::ServerParam::i().staminaMax() * 0.5 )
    {
        s_recover_mode = true;
    }
    else if ( wm.self().stamina() > rcsc::ServerParam::i().staminaMax() * 0.7 )
    {
        s_recover_mode = false;
    }

    /*--------------------------------------------------------*/
    const Vector2D ball_pos = wm.ball().inertiaPoint( std::min( self_min, std::min(opp_min, mate_min ) ) );

    double dash_power = 100.0;
    const double my_inc
        = wm.self().playerType().staminaIncMax()
        * wm.self().recovery();

    if ( wm.ourDefenseLineX() > wm.self().pos().x
         && ball_pos.x < wm.ourDefenseLineX() + 20.0 )
    {
        rcsc::dlog.addText( rcsc::Logger::TEAM,
                            __FILE__": getDashPower() correct DF line. keep max power" );
        // keep max power
        dash_power = rcsc::ServerParam::i().maxDashPower();
    }
    else if ( s_recover_mode )
    {
        dash_power = my_inc - 25.0; // preffered recover value
        if ( dash_power < 0.0 ) dash_power = 0.0;

        rcsc::dlog.addText( rcsc::Logger::TEAM,
                            __FILE__": getDashPower() recovering" );
    }
    // exist kickable teammate
    else if ( wm.existKickableTeammate()
              && wm.ball().distFromSelf() < 20.0 )
    {
        dash_power = std::min( my_inc * 1.1,
                               rcsc::ServerParam::i().maxDashPower() );
        rcsc::dlog.addText( rcsc::Logger::TEAM,
                            __FILE__": getDashPower() exist kickable teammate. dash_power=%.1f",
                            dash_power );
    }
    // in offside area
    else if ( wm.self().pos().x > wm.offsideLineX() )
    {
        dash_power = rcsc::ServerParam::i().maxDashPower();
        rcsc::dlog.addText( rcsc::Logger::TEAM,
                            __FILE__": in offside area. dash_power=%.1f",
                            dash_power );
    }
    else if ( ball_pos.x > 25.0
              && ball_pos.x > wm.self().pos().x + 10.0
              && self_min < opp_min - 6
              && mate_min < opp_min - 6 )
    {
        dash_power = rcsc::bound( rcsc::ServerParam::i().maxDashPower() * 0.1,
                                  my_inc * 0.5,
                                  rcsc::ServerParam::i().maxDashPower() );
        rcsc::dlog.addText( rcsc::Logger::TEAM,
                            __FILE__": getDashPower() opponent ball dash_power=%.1f",
                            dash_power );
    }
    // normal
    else
    {
        dash_power = std::min( my_inc * 1.7,
                               rcsc::ServerParam::i().maxDashPower() );
        rcsc::dlog.addText( rcsc::Logger::TEAM,
                            __FILE__": normal mode dash_power=%.1f",
                            dash_power );
    }

    return dash_power;
}
