// -*-c++-*-

/*
 *Copyright:

 Copyright (C) Hiroki SHIMORA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 3, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#include "puppets_player.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

using namespace rcsc;

/*-------------------------------------------------------------------*/
/*!

*/
#include "devel_field_evaluator.h"
#include "param_field_evaluator.h"

FieldEvaluator::ConstPtr
PuppetsPlayer::s_create_field_evaluator()
{
#if 0
    return FieldEvaluator::ConstPtr( new DevelFieldEvaluator );
#else
    return FieldEvaluator::ConstPtr( new ParamFieldEvaluator );
#endif
}


/*-------------------------------------------------------------------*/
/*!
*/
#include "actgen_cross.h"
#include "actgen_direct_pass.h"
#include "actgen_self_pass.h"
#include "actgen_strict_check_pass.h"
#include "actgen_short_dribble.h"
#include "actgen_simple_dribble.h"
#include "actgen_shoot.h"
#include "actgen_action_chain_length_filter.h"
#include "actgen_cooperative_offside_line_pass.h"

ActionGenerator::ConstPtr
PuppetsPlayer::s_create_action_generator()
{
    CompositeActionGenerator * g = new CompositeActionGenerator();

    //
    // shoot
    //
    g->addGenerator( new ActGen_RangeActionChainLengthFilter
                     ( new ActGen_Shoot(),
                       2, ActGen_RangeActionChainLengthFilter::MAX ) );

    //
    // cooperative offside line pass
    //
    g->addGenerator( new ActGen_CooperativeOffsideLinePass() );


    //
    // strict check pass
    //
    g->addGenerator( new ActGen_MaxActionChainLengthFilter
                     ( new ActGen_StrictCheckPass(), 1 ) );

    //
    // cross
    //
    g->addGenerator( new ActGen_MaxActionChainLengthFilter
                     ( new ActGen_Cross(), 1 ) );

    //
    // direct pass
    //
#if 0
    g->addGenerator( new ActGen_RangeActionChainLengthFilter
                     ( new ActGen_DirectPass(),
                       2, ActGen_RangeActionChainLengthFilter::MAX ) );
#else
    g->addGenerator( new ActGen_DirectPass() );
#endif

#if 1
    //
    // short dribble
    //
    g->addGenerator( new ActGen_MaxActionChainLengthFilter
                     ( new ActGen_ShortDribble(), 1 ) );
#endif

    //
    // self pass (long dribble)
    //
    g->addGenerator( new ActGen_MaxActionChainLengthFilter
                     ( new ActGen_SelfPass(), 1 ) );


    //
    // simple dribble
    //
    // g->addGenerator( new ActGen_RangeActionChainLengthFilter
    //                  ( new ActGen_SimpleDribble(),
    //                    2, ActGen_RangeActionChainLengthFilter::MAX ) );

    return ActionGenerator::ConstPtr( g );
}


/*-------------------------------------------------------------------*/
/*!
*/
FieldEvaluator::ConstPtr
PuppetsPlayer::createFieldEvaluator() const
{
    return s_create_field_evaluator();
}


/*-------------------------------------------------------------------*/
/*!
*/
ActionGenerator::ConstPtr
PuppetsPlayer::createActionGenerator() const
{
    return s_create_action_generator();
}
